/*
The Unreal Engine is protected under the GPL.
For more information, please see http://www.epicgames.com/ut2k4_eula.html

Use and distribution of Gamebots is dictated by the terms in the GNU GPL.
http://www.gnu.org/licenses/gpl.html

Copyright 2006,  Jessica Bayliss, Tim Garwood
*/

//////////////////////////////////////////
//
// BotClientConnection.uc: 
//
// Version: $Id$
//
// Revision: $Log$
//
/////////////////////////////////////////

class BotClientConnection extends TcpLink
	config(BotAPI);

////////////////////
//
// variables
//
////////////////////

var string ReceivedData;

// MAX_ARG_NUM is the maximum number of args in a message
// the arrays below should use it, except that the constant
// apparently doesn't work with the arrays!
const MAX_ARG_NUM = 9;
var string ReceivedArgs[9];
var string ReceivedVals[9];

// used to store info inside function where local wont work
// set property text dosent seem to work with a local
var actor tempActor;

// set true for verbose debug logs
var config bool bDebug;

// set true for iterative mode
var config bool bIterative;

// delimiters for strings sent to clients. set to match those of the server
// as = identifies the start of an attribute
var string as;
// ae = attribute end
var string ae;
// ib = item break
var string ib;

////////////////////
//
// Events
//
////////////////////

// Called whenever we receive a line of text.
// Parse into lines and call ReceivedLine
event ReceivedText( string Text )
{
  local int i;
  local string receivedString;
  local bool done;
  
  done = false;

 // if(bDebug)
 //   log("Received - "$Text);

  ReceivedData = ReceivedData $ Text;
  
  // remove a LF which arrived in a new packet
  // and thus didn't get cleaned up by the code below
  if(Left(ReceivedData, 1) == Chr(MAX_ARG_NUM+1))
    ReceivedData = Mid(ReceivedData, 1);

  i = InStr(ReceivedData, Chr(MAX_ARG_NUM+4));
  if (i == -1)
    done = true;

  while( !done )
  {
    receivedString = Left(ReceivedData, i);
    i++;
    // check for any LF following the CR.
    if(Mid(ReceivedData, i, 1) == Chr(MAX_ARG_NUM+1))
      i++;

    ReceivedData = Mid( ReceivedData, i );

    ReceivedLine( receivedString );

    if (LinkState != STATE_Connected) {
      done = true;
    } else {
      i = InStr(ReceivedData, Chr(MAX_ARG_NUM+4));
      if (i == -1)
	done = true;
    }
  }
}

////////////////////
//
// Functions
//
////////////////////

// returns the string for the argument value
// we could save some time if there was a direct
// mapping between names and slots
// then this loop wouldn't exist
function string GetArgVal(string argName)
{
  local string argValue;
  local int i;
  local bool done;

  i = 0;

  argValue = "";
  done = false;

  while (!done && (i < MAX_ARG_NUM) && (ReceivedArgs[i] != ""))
    {
      if (ReceivedArgs[i] ~= argName) {
	  argValue = ReceivedVals[i];
	 // log("when argName = " $argName$ " , argValue = " $argValue); 
	  done = true;
	}
      i++;
    }
  
  return argValue;
}


//Send a line to the client
function SendLine(string Text, optional bool bNoCRLF)
{
  if(bDebug)
    log("    Sending: "$Text);
  if(bNoCRLF)
    SendText(Text);
  else
    SendText(Text$Chr(13)$Chr(10));
}


//Concat two strings togeather with our list bookends
function string MakeItem(string first, string second)
{
  return (as$first$ib$second$ae);	
}

////////////////////
//
// Game Type Specific Functions:
// These would change if game types
// came from the same root class.
//
////////////////////

// This function returns the game status string
// which is dependent on the game type (game
// types don't derive from a common type,
// although they might in the future)
function string getGameStatus(String gameClass ) 
{
  local string returnString;

  // [work] commented out game types need to be added
  // back into the game  
  switch(gameClass)
  {
    case "BotTeamGame":
      returnString = BotTeamGame(Level.Game).GetGameInfo();
      break;
    case "BotDeathMatch":
      // [2003 change] this used to be BotDeathMatchPlus in regular UT
      returnString = BotDeathMatch(Level.Game).GetGameInfo();
      break;
    case "BotCTFGame":
      returnString = BotCTFGame(Level.Game).GetGameInfo();
      break;
      //added 2-6-06  Tim Garwood
      //ut2003 uses double domination
    case "BotDoubleDomGame":
      returnString = BotDoubleDomGame(Level.Game).GetGameInfo();
      break;
      //added 2-9-06  Tim Garwood
      //for bombing run game in ut2003.
    case "BotBombingRunGame":
      returnString = BotBombingRunGame(Level.Game).GetGameInfo();
      break;
    default:
      returnString = "No defined game type for this game is available.";
      break;
  }

  return returnString;  
}


// Sends out the game state info
// There's no clean way to do this since gametypes are not derived from
//a common ancestor that has the right function.
function sendGameStateFromGameType( RemoteBot theBot, String gameClass ) 
{
  // [work] other game types need to be implemented before being
  // uncommented
  switch(gameClass)
    {
    case "BotTeamGame":
      sendLine( BotTeamGame(Level.Game).GetGameStatus() );
      break;
    case "BotDeathMatch":
	// [2003 change] used to be BotDeathMatchPlus
      sendLine( BotDeathMatch(Level.Game).GetGameStatus() );
      break;
      //ut2003 change - Tim Garwood 2-6-06
      //Domination is now DoubleDomination in '03
    case "BotDoubleDomGame":
      sendLine( BotDoubleDomGame(Level.Game).GetGameStatus() );
      break;
    case "BotCTFGame":
      sendLine( BotCTFGame(Level.Game).GetGameStatus(theBot) );
      break;
      //2-9-06  Tim Garwood
      //for bombing run game in ut2003
    case "BotBombingRunGame":
      sendLine( BotBombingRunGame(Level.Game).GetGameStatus() );
      break;
    }
}
