/*
The Unreal Engine is protected under the GPL.
For more information, please see http://www.epicgames.com/ut2k4_eula.html

Use and distribution of Gamebots is dictated by the terms in the GNU GPL.
http://www.gnu.org/licenses/gpl.html

Copyright 2006,  Jessica Bayliss, Tim Garwood
*/

//////////////////////////////////////////
//
// BotConnection.uc: 
//
// Version: $Id: BotConnection.uc,v 1.2 2003/06/22 19:58:42 jdb Exp jdb $
//
// Revision: $Log: BotConnection.uc,v $
// Revision: Revision 1.2  2003/06/22 19:58:42  jdb
// Revision: A lot of commenting and cleanup along with
// Revision: some comparison of the ut2003 code for bot connection.
// Revision: Next task: refactor so that all client connections inherit
// Revision: from the same class.
// Revision:
//
/////////////////////////////////////////

class BotConnection extends BotClientConnection;

////////////////////
//
// variables
//
////////////////////

// delay between visionUpdates
var config float visionTime;

// used to store info inside function where local wont work
// set property text dosent seem to work with a local
var bool tempBool;

// the one server and the actual bot we're in control of
var BotServer Parent;
var RemoteBot theBot;

// set true for iterative mode
//var config bool bIterative;

// [work] should the following be added
// var int attrNum;

////////////////////
//
// Events
//
////////////////////

// triggered when a socket connection with the server is established
event Accepted()
{
  local string gameInfoStr, levelName;
  local int i;
  
  if(bDebug)
    log("Accepted BotConnection" @ self);
  
  gameInfoStr = getGameStatus(Parent.gameClass);
  levelName = string(Level);
  i = InStr(Caps(levelName), ".LEVELINFO");
  if(i != -1)
    levelName = Left(levelName, i);

  SendLine("NFO" $ib$as$ "Gametype" $ib$ Parent.gameClass $ae$ib$as$ 
	   "Level" $ib$ levelName $ae$ gameInfoStr);
  gotoState('monitoring','WaitingForInit');
}

// triggered when we've gotten the init message from the client
event InitReceived() 
{
  local string clientName;
  local string teamString;
  local int teamNum;
  local int DONTCARE;
  // [2003 new] start location, rotation, and class name
  local vector startLocation;
  local rotator startRotation;
  local string className; 

  DONTCARE = 255;

  // set up the name/team
  clientName = GetArgVal("Name");
  ParseVector(startLocation,"Location");
  ParseRot(startRotation);
  className = GetArgVal("ClassName");

  teamString = GetArgVal("Team");
  if( teamString == "" ) {
	//log("teamString is empty in initReceived() in BotConnection.uc");
    teamNum = DONTCARE;
   }
  else {
//	log("teamstring was valid in initReceived in BotConnection.uc");
    teamNum = int(teamString);
  }
  // add the bot into the game
  addBotToGameType(clientName, teamNum, startLocation, startRotation, className);
    	
  // there used to be code for broadcasting a welcome message here
  // this is now handled by the game type itself
    
  if(bDebug) {
    if(theBot != None)
      log("added bot, now running");
    else log("ERROR ADDING BOT");
  }
	
  gotoState('monitoring','Running');
}

// triggered when the client closes the connection
// [2003 change] since the controller is now separate from
// the pawn destruction has changed
event Closed()
{
   local Pawn tempPawn;

    if ( theBot != None && theBot.Pawn != None ) {
        theBot.SetLocation(theBot.Pawn.Location);
        theBot.Pawn.RemoteRole = ROLE_SimulatedProxy;
        theBot.Pawn.UnPossessed();
        tempPawn = theBot.Pawn;
        theBot.Pawn = None;
        tempPawn.Destroy();
    }

    if(theBot != None) {
        theBot.Destroy();
    }
    /* Revision 12-8-05  Tim Garwood
    Decremented the number of players in the game so the server browser in ut2003
    accurately reflects the # of players...
    */
    
    if( Level.Game.isA('BotDeathMatch')) {
        BotDeathMatch(Level.Game).setnumplayers( BotDeathMatch(Level.Game).getnumplayers() - 1);
        BotDeathMatch(Level.Game).setnumremotebots( BotDeathMatch(Level.Game).getNumRemoteBots() - 1);
    } else if( Level.Game.isA('BotTeamGame')) {
        BotTeamGame(Level.Game).setnumplayers( BotTeamGame(Level.Game).getnumplayers() - 1);
        BotTeamGame(Level.Game).setnumremotebots( BotTeamGame(Level.Game).getNumRemoteBots() - 1 );
    } else if( Level.Game.isA('BotCTFGame')) {
        BotCTFGame(Level.Game).setNumPlayers( BotCTFGame(Level.Game).getNumPlayers() - 1);
        BotCTFGame(Level.Game).setnumremotebots( BotCTFGame(Level.Game).getNumRemoteBots() - 1 );
    } else if( Level.Game.isA('BotDoubleDomGame')) {
        BotDoubleDomGame(Level.Game).setNumPlayers( BotDoubleDomGame(Level.Game).getNumPlayers() - 1);
        BotDoubleDomGame(Level.Game).setnumremotebots( BotDoubleDomGame(Level.Game).getNumRemoteBots() - 1 );
    } else if( Level.Game.isA('BotBombingRunGame')) {
        BotBombingRunGame(Level.Game).setNumPlayers( BotBombingRunGame(Level.Game).getNumPlayers() - 1);
        BotBombingRunGame(Level.Game).setnumremotebots( BotBombingRunGame(Level.Game).getNumRemoteBots() - 1 );
    }

    Destroy();
}


////////////////////
//
// Functions
//
////////////////////

// Called automagically when the game starts
function PostBeginPlay()
{
  Parent = BotServer(Owner);
  ib = Parent.ib;
  as = Parent.as;
  ae = Parent.ae;
  
  if(bDebug)
    log("Spawned BotConnection");
  // problem ???
  SaveConfig();
}

//Here is where we handle incoming commands
// Commands expected to look like:
// {RUN {Argument value} {Arg value}...}
// Currently hard coded to take no more than MAX_ARG_NUM args
// Command type and arguments can be
// any length, but first space terminates the name. Values can
// have spaces or any other kind of character.
function ReceivedLine(string line)
{    
  local string cmdType, argBody, rem;
  local int endloc, wordsplit, attrNum;
  local name curBotState;

  // return if we're done
  if( theBot != none ) {
    curBotState = theBot.GetStateName();
    if( curBotState == 'Dying' || curBotState == 'GameEnded')
      return;
  }

  if(bDebug)
    log(line);
  
  wordsplit = InStr(line,ib);
  if( wordsplit == -1)
    wordsplit = Len(line);
  
  cmdType = left(line,wordsplit);	
  rem = mid(line,InStr(line,as));
  attrNum = 0;
  
  // iterate through attr/val pairs, storing them in the
  // parallel arrays ReceivedArgs and ReceivedVals
  while(attrNum < MAX_ARG_NUM && rem != "")
    {
      endloc = InStr(rem,ae);
      argBody = mid(rem,1,(endloc - 1));
      
      wordsplit = InStr(argBody,ib);
      ReceivedArgs[attrNum] = left(argBody,wordsplit);
      ReceivedVals[attrNum] = mid(argBody,(wordsplit + 1));
  
      rem = mid(rem,1); //advance
      rem = mid(rem,InStr(rem,as));
      attrNum++;
    }
	
  cmdType = Caps(cmdType);  
  ProcessAction(cmdType);
}


// should use int's for locations rather than floats
// we don't need to be that precise
function ParseVector(out vector v, string vecName)
{
  local int i;
  local string rem;
  local string delim;
  
  delim = ib;
  
  rem = GetArgVal(vecName);
  if(rem != "")
    {
      if( InStr(rem,delim) == -1 )
	delim = ",";
      i = InStr(rem,delim);
      v.X = float(left(rem,i));
      rem = mid(rem,i+1);
      i = InStr(rem,delim);
      v.Y = float(left(rem,i));
      v.Z = float(mid(rem,i+1));
    }
  else
    {
      v.x = float( GetArgVal("x") );
      v.y = float( GetArgVal("y") );
      v.z = float( GetArgVal("z") );
    }
}

// again, we should use int's rather than floats
// although we can make them into floats anyway
function ParseRot(out rotator rot)
{
  local int i;
  local string rem;
  local string delim;

  delim = ib;
  
  rem = GetArgVal("Rotation");
  if(rem != "")
    {
      if( InStr(rem,delim) == -1 )
	  delim = ",";
      i = InStr(rem,delim);
      rot.Pitch = float(left(rem,i));
      rem = mid(rem,i+1);
      i = InStr(rem,delim);
      rot.Yaw = float(left(rem,i));
      rot.Roll = float(mid(rem,i+1));
    }
  else
    {
      rot.Pitch = float( GetArgVal("pitch") );
      rot.Yaw = float( GetArgVal("yaw") );
      rot.Roll = float( GetArgVal("roll") );
    }
}

// the commands themselved should be enumerated
// runto never actually sets running! (bIsWalking?)
// runto is really a go to location command
// perhaps strafe should set the rotation to the
// current rotation? - all issues in RemoteBot
function ProcessAction(string cmdType)
{
  local Controller C;
  local int i;
  local vector v, v2;
  local bool boolResult;
  local string target, id, outBuf, focusTarg;
  // [2003 change] added r2, speed, and accel
  local rotator r, r2;
  local float speed;
  local float accel;

  //for chopping up the incoming target ID
  local string tempString;
  local string idNumString;
  
  if( theBot == none )
    log("BOT IS NULL IN PROCESSACTION");
  else if( thebot.pawn == none )
	log("PAWN IS NULL IN PROCESSACTION");

  if(bDebug)
    log("comandType:"@cmdType);

  // [2003 change] pauser is now an object rather than a string
  if (bIterative)
    Level.Pauser=None;

  speed = float(GetArgVal("Speed"));

  switch(cmdType)
    {
    case "RUNTO":
      speed = float(GetArgVal("Speed"));
	if(speed != 0.0) {
        if(theBot.Physics == PHYS_Flying) {
	    theBot.Pawn.airspeed = speed;
		
	  }
	  else if(theBot.Physics == PHYS_Swimming) {
	    theBot.Pawn.waterspeed = speed;
	  }
        else {
	    theBot.Pawn.groundspeed = speed;
	  }
	}

	accel = float(GetArgVal("Accel"));
	if(accel != 0.0) {
	  theBot.Pawn.AccelRate = accel;
	}

      target = GetArgVal("Target");
      if(target == "") 
	{
	  ParseVector(v,"Location");
	  theBot.myDestination = v;

          //added 2-6-06 Tim Garwood
          theBot.Destination = v;

	  //theBot.RemoteDestination = none;
	  theBot.GotoState('Startup', 'Move');
	}
      else
	{
	  SetPropertyText("tempActor",target);

	  if( tempActor != none &&
	      theBot.FastTrace(tempActor.Location) &&
	      theBot.inFront(tempActor.Location) )
	    {
	      //theBot.RemoteDestination = none;
	      //zero out myDestination
	      theBot.myDestination = tempActor.Location;
	      
	      //added 2-6-06  Tim Garwood
	      theBot.Destination = tempActor.Location;

	      theBot.GotoState('Startup', 'Move');
	    }
	}
      break;
    case "TURNTO":
      target = GetArgVal("Target");

      if(target == "")
	{
	  ParseRot(r);
	  if(r.Yaw == 0 && r.Pitch == 0 && r.Roll == 0)
	    {
	      //no target or rotation defined
	      ParseVector(v,"Location");
	      theBot.myFocus = v;

	      //added 1-23-06 Tim Garwood - helps bot aiming??
	      theBot.focalPoint = v;


	      theBot.RemoteFocus = None;
	    }
	  else
	    {

	      //no target, no location, yes rotation
	      theBot.myRotation = r;

              //added 1-23-06  Tim Garwood - helps bot aiming??
              theBot.Pawn.SetViewRotation( r );
	      theBot.setRotation( r );

		//why is this commented out? - 1-23-06  TG
		//theBot.myFocus = theBot.Pawn.Location + ( vector(r) * 1000);

		theBot.myFocus = vect(0,0,0);
		theBot.RemoteFocus = none;

	    }
	}
      else
	{

	  //target defined
	  SetPropertyText("tempActor",target);


	  //go through the controller list and find the controller that we are facing
	  for(C = Level.ControllerList; C != None; C = C.NextController ) {

		//if the unique player ID of the current controller is the one we want, set up
		//our target information
		if( theBot.CanSee( C.Pawn ) && theBot.Pawn.LineOfSightTo( C.Pawn )) {
		    if( (C$C.PlayerReplicationInfo.PlayerID) == target ) {
			//point bot at the target they specified. 1-25-06  Tim Garwood
                        theBot.myFocus = C.Pawn.Location;
                        theBot.RemoteFocus = C.Pawn;
                        theBot.Enemy = C.Pawn;
			theBot.RemoteEnemy = C;
			theBot.target = C.Pawn;
			theBot.focus = C.Pawn;
			theBot.focalPoint = C.Pawn.Location;
			//theBot.setRotation( Rotator(C.Pawn.Location));
			break;
		   }
		}
	  }

	}

	if (GetArgVal("RotationRate")!="") {
        ParseRot(r2);
        theBot.myRotationRate = r2;
	}

      theBot.StopWaiting();
      theBot.GotoState('Startup', 'Turning');
      break;
    case "ROTATE":

      target = GetArgVal("Axis");
      r = theBot.Pawn.Rotation;
      i = int(GetArgVal("Amount"));
      if(target == "Vertical")
	{
	  r.Pitch = r.Pitch + i;

	}
      else
	{
	  r.Yaw = r.Yaw + i;

	}
	if (GetArgVal("RotationRate")!="") {
        ParseRot(r2);
        theBot.myRotationRate = r2;
	}

      theBot.myRotation = r;
      theBot.myFocus = theBot.Pawn.Location + ( vector(r) * 1000);

      //added 1-20-06  Tim Garwood - helps bot aiming??
      theBot.myTarget = theBot.myFocus;

      //added 1-23-06  Tim Garwood - helps bot aiming??
      theBot.focalPoint = theBot.myFocus;

      //added 1-24-06  Tim Garwood - helps bot aiming?
      theBot.setRotation( r );
      theBot.UpdatePawnViewPitch();

      theBot.RemoteFocus = None;
      theBot.StopWaiting();
      theBot.GotoState('Startup', 'Turning');
      break;

    case "NOP":
      // just let's us know the client is alive
      // no response required
      break;
    case "JUMP":
      theBot.RemoteJump();
      break;
    case "STRAFE":

	speed = float(GetArgVal("Speed"));
      if(speed != 0.0) {
        if(theBot.Pawn.Physics == PHYS_Flying) {
          theBot.Pawn.airspeed = speed;
        } else if(theBot.Physics == PHYS_Swimming) {
	    theBot.Pawn.waterspeed = speed;
	  } else {
          theBot.Pawn.groundspeed = speed;
        }
      }

	accel = float(GetArgVal("Accel"));
	if(accel != 0.0) {
	  theBot.Pawn.AccelRate = accel;
	}

      ParseVector(v,"Location");
      theBot.myDestination = v;

      //added 1-25-06  Tim Garwood
      theBot.Destination = v;

      focusTarg = GetArgVal("Target");

     if(focusTarg == "")
	{
          //no target, but we have a location to look at.
          //should we set Enemy, RemoteEnemy, etc. to 'none' since we aren't looking at any
          //particular bot?
	 /* MOVETO ONLY TAKES ACTORS FOR A FOCAL POINT. */
            //1-27-06 the issue remains - no way to make a bot strafe while looking at a location
            //we need an ID of an actor to look for.
          ParseVector(v2,"Focus");
	  theBot.myFocus = v2;
	  theBot.myTarget = v2;
	  theBot.RemoteFocus = none;

	  theBot.focalPoint = v2;
	 // theBot.Focus.SetLocation( v2 );

	  //limits us to using vectors.
	  theBot.setRotation( Rotator( v2 ));

	}
      else
	{
	 //we have a target. point at it.
          for( C = Level.ControllerList; C != None; C = C.NextController) {
           if( (C$C.PlayerReplicationInfo.PlayerID) == focusTarg) {

               //need to be able to see the target first...
               if( theBot.CanSee( C.Pawn ) && theBot.Pawn.LineOfSightTo(C.Pawn)) {
                //point the bot at the location of the target
                theBot.myFocus = C.Pawn.Location;
                theBot.RemoteFocus = C.Pawn;
                theBot.Enemy = C.Pawn;
                theBot.RemoteEnemy = C;
                theBot.target = C.Pawn;
                theBot.focus = C.Pawn;
                theBot.focalPoint = C.Pawn.Location;
                theBot.setRotation( Rotator(C.Pawn.Location));
                break;
               }
           }
          }
         }




        /*
	  SetPropertyText("tempActor",focusTarg);
	  //focusTarg is set propertly, but why does tempActor get 'none' after SetProperyText? - 1.25.06 Tim Garwood
	  log("In strafe in BotConnection.uc - focusTarg is " $ focusTarg );
	  log("In strafe in BotConnection.uc - tempActor has been set to " $ tempActor );

	  if( tempActor != none &&
	      theBot.FastTrace(tempActor.Location) &&
	      theBot.inFront(tempActor.Location) )
	    {
	      theBot.RemoteFocus = tempActor;
	      //zero out myFocus
	      theBot.myFocus = v2;
	    }
	} */
      theBot.StopWaiting();
      theBot.GotoState('Startup', 'Strafe');
      break;
    case "SHOOT":


      target = GetArgVal("Target");

      if( target == "") {
       //we are shooting at a location. Should we set Enemy,RemoteEnemy, etc to 'none'
       //since we don't have a specific target? 1-26-06 - Tim Garwood
       ParseVector(v,"Location");
       theBot.SetPropertyText("Enemy",target);
       theBot.SetPropertyText("RemoteEnemy",target);
       theBot.StopWaiting();
       theBot.myTarget = v;
       
       //added 2-6-06  Tim Garwood - helps bot aiming?
       theBot.myFocus = v;
       theBot.focalPoint = v;
       theBot.setRotation( Rotator(v));
       //changed 2-16-06   Tim Garwood
       //set Enemy and Target of this bot to 'none' so we know that we are just shooting
       //at a location in RemoteBot.AdjustAim
       theBot.Enemy = none;
       theBot.Target = none;

      }

      else {
        for( C=Level.ControllerList; C != None; C = C.NextController) {

           if( (C$C.PlayerReplicationInfo.PlayerID) == target ) {

               //look at this target and set it to be our enemy
               theBot.myFocus = C.Pawn.Location;
               theBot.RemoteFocus = C.Pawn;
               theBot.Enemy = C.Pawn;
               theBot.RemoteEnemy = C;
               theBot.target = C.Pawn;
               theBot.focus = C.Pawn;
               theBot.focalPoint = C.Pawn.Location;
               theBot.setRotation( Rotator(C.Pawn.Location));
               break;

           }
        }
      }
           

      target = GetArgVal("Alt");
      SetPropertyText("tempBool",target);
      theBot.RemoteFireWeapon(tempBool);
      
      break;
    case "STOPSHOOT":

      theBot.StopWaiting();
      theBot.HaltFiring();

      break;
    case "TRACK":

      target = GetArgVal("Target");
      if(target != "") 
	{
	  SetPropertyText("tempActor",target);
	  if( theBot.FastTrace(tempActor.Location) && theBot.inFront(tempActor.Location) )
	    {
	      theBot.RemoteFocus = tempActor;
	      theBot.StopWaiting();
	      theBot.GotoState('Startup', 'Tracking');
	    }
	}
      break;
    case "STOP":

      theBot.GotoState('Startup', 'Begin');
      break;

    case "CHANGEWEAPON":
      target = GetArgVal("ID");
      if( target ~= "best" )
	{
	  theBot.SwitchToBestWeapon();
	}
      else
	{
	  SetPropertyText("tempActor",target);

	  if(tempActor != none)
	    {
	      theBot.pawn.PendingWeapon = Weapon(tempActor);
	      if ( theBot.Pawn.Weapon == None )
		theBot.ChangedWeapon();
	      else if ( theBot.Pawn.Weapon != theBot.Pawn.PendingWeapon )
		theBot.Pawn.Weapon.PutDown();
	    }
	}
      break;
    case "CHECKREACH":

      target = GetArgVal("Target");
      id = GetArgVal("id");
      if(target == "")
	{
	  ParseVector(v,"Location");
	  //2-24-06  Tim Garwood changed to Pawn.PointReachable(v) because pawn's position 
	  //is what is important.
	  boolResult = theBot.pointReachable(v);
	  sendLine("RCH" $ib$as$ "ID" $ib$ id $ae$ib$as$ "Reachable" $ib$ boolResult
		   $ae$ib$as$ "From" $ib$ theBot.Pawn.Location $ae);
	}
      else
	{
	  //FIXME
	  //setPropertyText doesn't seem to be working. If we are trying to reach a player, this
	  //loop seems to work. Not sure what to do about other things like pickups. Probably better
	  //to send a location.
	  foreach AllActors( class 'Controller', C) {

	          if( (C $ C.PlayerReplicationInfo.PlayerID) == target )
	           break;

          }

          //SetPropertyText("tempActor",target);

	  if( C != none &&
	      //theBot.FastTrace(tempActor.Location) &&
              theBot.Pawn.FastTrace( C.Pawn.Location ) &&
              theBot.inFront( C.Pawn.Location) )
	    {
	      boolResult = theBot.actorReachable( C.Pawn );
	      sendLine("RCH" $ib$as$ "ID" $ib$ id $ae$ib$as$ "Reachable" $ib$ boolResult
		       $ae$ib$as$ "From" $ib$ theBot.Pawn.Location $ae$ib$as$ "To" $ib$ C.Pawn.Location $ae);
	    }
	}
      break;
    case "SETWALK":

      target = GetArgVal("Walk");
      SetPropertyText("tempBool",target);
      theBot.Pawn.bIsWalking = tempBool;
      break;
    case "SETPHYS":

      // set up what the bot is doing physically
      target = GetArgVal("Physics");
      target = Caps(target);
	switch(target) {
	  case "WALKING":
	    theBot.Pawn.SetPhysics(PHYS_Walking);
          if (bDebug) {
	      log("Setting Physics Walking");
	      if ( theBot.Physics == PHYS_Walking) {
	         log("Bot already walking");
		}
          }
	    break;
	  case "FALLING":

	    theBot.Pawn.SetPhysics(PHYS_Falling);
	    break;
	  case "SWIMMING":

	    theBot.Pawn.SetPhysics(PHYS_Swimming);
	    break;
	  case "FLYING":

	    theBot.Pawn.SetPhysics(PHYS_Flying);
          if (bDebug) {
	      log("Setting Physics Flying");
		if(theBot.Pawn.Physics == PHYS_Flying) {
		  log("Bot already flying");
		}
	    }
          break;
	  case "ROTATING":

	    theBot.Pawn.SetPhysics(PHYS_Rotating);
	    break;
	  case "PROJECTILE":

	    theBot.Pawn.SetPhysics(PHYS_Projectile);
	    break;
	  case "NONE":

	  case "":
	    theBot.Pawn.SetPhysics(PHYS_None);
	    break;
	  }
	  break;

    case "GETPATH":
      //clear the old path
      for ( i=0; i<16; i++ )
	{
	  if ( theBot.RouteCache[i] == None )
	    break;
	  else
	    {
	      theBot.RouteCache[i] = None;
	    }
	}
      ParseVector(v,"Location");
      theBot.FindPathTo(v);
      id = GetArgVal("id");
      outBuf = "PTH"$ib$as$"ID"$ib$id$ae;
      for ( i=0; i<16; i++ )
	{
	  if ( theBot.RouteCache[i] == None )
	    break;
	  else
	    {
	      outBuf = outBuf$ib$as$i$ib$theBot.RouteCache[i]$ib$theBot.RouteCache[i].Location$ae;
	    }
	}
      SendLine(outBuf);
      break;
    case "MESSAGE":
      //Note - currently only allow messages under 256 chars
      
      //UPDATE 2-13-06  Tim Garwood
      //added 2 functions (SendMessageToEveryone, SendMessageToTeam) in RemoteBot.uc
      //global and team messages can now be received by both human and bot players.

      target = GetArgVal("String");
      boolResult = bool(GetArgVal("Global"));
      if(target != "")
	{
	  if(boolResult) {
	    // this is a global message
	    // we wrote it with the @ sign meaning "global"
	    theBot.TeamBroadcast("@"$target);
          } else {
	    // this is a local message to the team
            theBot.TeamBroadcast(target);
          }
	}
      break;
    case "PING":

      SendLine("Pong");
      break;
    case "TEST":

      theBot.GotoState('Startup', 'Test');
      break;
    case "INIT":
      InitReceived();
      break;
    case "INCH":
 
      //test function
      /* must deal with target, focus and destination */
      theBot.StopWaiting();
      theBot.myDestination = (100 * vector(theBot.Pawn.Rotation)) + theBot.Pawn.Location;
      theBot.GotoState('Startup', 'Move');
      break;
    }
}


//Send a line to the client
function SendLine(string Text, optional bool bNoCRLF)
{
  if(bDebug)
    log("    Sending: "$Text);
  if(bNoCRLF)
    SendText(Text);
  else
    SendText(Text$Chr(13)$Chr(10));
}


////////////////////
//
// Game Type Specific Functions:
// These would change if game types
// came from the same root class.
//
////////////////////

// adds a bot to a particular game type
// There's no clean way to do this since gametypes are not derived from
//a common ancestor that has AddRemoteBot.
function addBotToGameType(string clientName, int teamNum, vector startLocation, rotator startRotation, string className )
{

  // [work] the following needs various game types to be defined to
  // start uncommenting blocks
  switch(Parent.gameClass)
    {
      case "BotTeamGame":
	  theBot = BotTeamGame(Level.Game).AddRemoteBot(self, clientName, teamNum, startLocation, startRotation, className);
	  break;
      // [2003 change] used to be BotDeathMatchPlus
      case "BotDeathMatch":
	theBot = BotDeathMatch(Level.Game).AddRemoteBot(self, clientName, teamNum, startLocation, startRotation, className);
	break;
     //2-6-06  Tim Garwood
     //ut2003 change - Domination is now DoubleDomination
      case "BotDoubleDomGame":
	theBot = BotDoubleDomGame(Level.Game).AddRemoteBot(self, clientName, teamNum, startLocation, startRotation, className);
	break;
      case "BotCTFGame":
	//theBot = BotCTF(Level.Game).AddRemoteBot(self, clientName, teamNum);
	theBot = BotCTFGame(Level.Game).AddRemoteBot(self, clientName, teamNum, startLocation, startRotation, className);
        break;
      case "BotBombingRunGame":

	theBot = BotBombingRunGame(Level.Game).AddRemoteBot(self, clientName, teamNum, startLocation, startRotation, className);
        break;
    }
}


////////////////////
//
// States
//
////////////////////

//fire right up into the loop for sending updates
auto state monitoring
{

Begin:
WaitingForInit:
  sleep(5.0);
  goto 'WaitingForInit';

  //running just means that the BotConnection is actively checking the game state
Running:

  if(theBot != None && Level.Pauser == None && !theBot.IsInState('GameEnded') && !theBot.IsInState('Dying'))
  {
    SendLine("BEG" $ib$as$ "Time" $ib$ Level.TimeSeconds $ae);
    // send the game's info
    sendGameStateFromGameType(theBot, Parent.gameClass);
    // send self info
    theBot.checkSelf();
    // send what can be seen
    theBot.checkVision();
    SendLine("END" $ib$as$ "Time" $ib$ Level.TimeSeconds $ae);
  }
  if (bIterative)
    Level.Pauser=theBot.PlayerReplicationInfo;
  
  sleep(visionTime);
  goto 'Running';
}


////////////////////
//
// Default properties
//
////////////////////

defaultproperties
{
  visionTime=0.20000
  AS="{"
  ae="}"
  ib=" "
}
