/*
The Unreal Engine is protected under the GPL.
For more information, please see http://www.epicgames.com/ut2k4_eula.html

Use and distribution of Gamebots is dictated by the terms in the GNU GPL.
http://www.gnu.org/licenses/gpl.html

Copyright 2006,  Joe Manolovich, Jessica Bayliss, Tim Garwood
*/

//////////////////////////////////////////
//
// DeathMatch.uc: 
//
// Version: $Id$
//
// Revision: $Log$
//
/////////////////////////////////////////

//class BotDeathMatch extends DeathMatch
//	config(botAPI);

/**** REVISION 12-1-05   Tim Garwood *****/
/**** changed config(botapi) to config(aiapi) to match the aiapi package ***/

class BotDeathMatch extends DeathMatch
	config(BotAPI);
	
#exec OBJ LOAD FILE=TeamSymbols_UT2003.utx
#exec OBJ LOAD FILE=TeamSymbols.utx				// needed right now for Link symbols, etc.
#exec OBJ LOAD File=AnnouncerMain.uax

////////////////////
//
// variables
//
////////////////////

var VizServer 		    theVizServer;
var BotServer 		    theBotServer;
var bool 		    bServerLoaded;
var RemoteBotInfo           RemoteBotConfig;
var int 		    NumRemoteBots;
var GameTypeHelper          game;

////////////////////
//
// Functions
//
////////////////////

// done
// automagically called
function PostBeginPlay()
{
  Super.PostBeginPlay();

  // set up the two servers
  if(!bServerLoaded) {
      game = spawn(class'GameTypeHelper',self);
      game.setupGameTypeHelper("BotDeathMatch");       
      theVizServer = spawn(class'VizServer',self);
      theVizServer.gameClass ="BotDeathMatch" ;
      theBotServer = spawn(class'BotServer',self);
      theBotServer.gameClass ="BotDeathMatch" ;

      bServerLoaded = true;
    }

  // set up the config for remote bots
  RemoteBotConfig = spawn(class'RemoteBotInfo');
  // [work] not sure if this is in the right place
  RemoteBotConfig.Difficulty = AdjustedDifficulty;
}


// [2003 change] fragLimit is now GoalScore
// returns a string of game information
function string GetGameInfo() {	
  local string outstr;
  local string ib, as, ae; 

  ib = theBotServer.ib; 
  as = theBotServer.as;
  ae = theBotServer.ae; 

  outStr = (ib$as$ "FragLimit" $ib$ GoalScore $ae$ib$as$ "TimeLimit" $ib$ timeLimit $ae);  
  return outStr;
}

// done
// return a formatted string of the game status
function string GetGameStatus() {
  local string outStr;
  local string as, ib; 

  as = theBotServer.as;
  ib = theBotServer.ib; 


  outStr = "GAM";    
  outStr = (outStr $ib$ game.GetPlayerScores( Level.ControllerList, theBotServer ) );
  return outStr;					
}

//Revision 1-17-06  Tim Garwood
//the other RestartPlayer function only spawned the bot if it was in the dying state, but we call
//this function from the dead state. restart the bot regardless of what state it is in.

function RestartPlayer( Controller aPlayer ) {

 
 if( aPlayer.isA('RemoteBot'))
	SpawnPlayer( RemoteBot(aPlayer));
 else
    Super.RestartPlayer( aPlayer );
}

/*
* Revision 12-8-05  Tim Garwood
Overriding the getnumplayers() method from UnrealMPGameInfo, maybe this
will accurately reflect the number of players in the game

also, added a bunch of accessors/mutators as seen below, i hope they
are self explanatory => they are used in BotConnection.uc when the connection
is closed.
*/

function int getnumplayers() {

	return numplayers;

}

function setnumplayers( int newplayers ) {

	numplayers = newplayers;
	minplayers = numplayers;
	
}

function int getnumremotebots() {

	return numremotebots;
	
}

function setnumremotebots( int newbots ) {

	numremotebots = newbots;
	
}



// Add a client bot to the game if possible else return None
function RemoteBot AddRemoteBot(BotConnection theConnection, string clientName, int teamNum, vector startLocation, rotator startRotation, string className) {
  
  local RemoteBot newBot;
  local NavigationPoint startSpot;

  startSpot = FindPlayerStart(NewBot);

  // try to spawn a client bot and return immediately if fail
  newBot = SpawnRemoteBot(startSpot, theConnection, startLocation, startRotation);
  
  if ( newBot == None ) {
    log("Failed to spawn remote bot.");
    return None;
  }
  
  // increment appropriate numbers since the bot exists
  NumRemoteBots++;
  NumPlayers++;
  
  //Revision 1-17-06  Tim Garwood
  //added this here so that no server bots enter the game AND so that
  //when a bot dies he doesn't "re-enter" the game, he just gets a new start spot.
  //this is because the engine checks to see if the player who died is a bot and if there
  //are too many players in the game, then it will destroy the bot. we don't want that.
  minPlayers = numPlayers;

  // set up names - there are no teams in deathmatch
  if(clientName == "") {
    newBot.PlayerReplicationInfo.PlayerName = "" $ newBot;
  } else {
    newBot.PlayerReplicationInfo.PlayerName = clientName;
  }
  
  changeName( newBot, clientName, true );

  if(className != "") {
    newBot.PawnClass = class<Pawn>(DynamicLoadObject(className,class'Class'));
  } else if(newBot.PawnClass == None) {
    log("newBot.PawnClass is None so its and xPawn");
    newBot.PawnClass = class<Pawn>(DynamicLoadObject("xGame.xPawn",class'Class'));
  }

  SpawnPlayer(newBot, startLocation, startRotation);
  return NewBot;
}

// Used by AddRemoteBot - does the actual bot spawning and if fails
// (maybe from a lack of starting
// positions), then returns None
function RemoteBot SpawnRemoteBot(NavigationPoint StartSpot, BotConnection theConnection, vector startLocation, rotator startRotation)
{
  local RemoteBot newBot;
  local actor tempbot;

  //newbot = Spawn(class'BotAPI.RemoteBot',self,,startLocation,startRotation);


  /****  REVISION:   11-30-05     Tim Garwood  ****/
  /****  The above line of code did not compile, it was generating a type mismatch ***/
  /****  so instead just generate a temp bot actor object and then immediately cast ***/
  /**** it into a RemoteBot and store it in newbot **/

  tempbot = Spawn(class'BotAPI.RemoteBot',self,,startLocation,startRotation);
  newbot = RemoteBot( tempbot );
  newBot.bIsPlayer = true;
  newBot.bHidden = false;

  if ( newBot != None )
  {
    //hook up connection to socket
    NewBot.myConnection = theConnection;
    NewBot.as = theConnection.as;
    NewBot.ae = theConnection.ae;
    NewBot.ib = theConnection.ib;

    // Set the player's ID.
    NewBot.PlayerReplicationInfo.PlayerID = CurrentID++;
    
    //RemoteBotConfig.CHIndividualize(NewBot, NewBot.PlayerReplicationInfo.PlayerID);

  }
  return NewBot;
}


// [2003 new]
// needs to be in the game rather than the helper to spawn
// correctly
function SpawnPlayer(RemoteBot newBot, optional vector startLocation, optional rotator startRotation) {
  local NavigationPoint startSpot;

  if(NewBot == None) {
    log("NewBot was none");
    return;
  }

  if(NewBot.Pawn != None) {
    log("Already had pawn");
    return;
  }

  startSpot = FindPlayerStart(NewBot);

  if(startLocation == vect(0,0,0)) {
    newBot.Pawn = Spawn(GetDefaultPlayerClass(NewBot),,,StartSpot.Location,StartSpot.Rotation);
  } else {
     NewBot.Pawn = Spawn(NewBot.PawnClass,,,startLocation,startRotation);
  }
  //  log("PawnClass: "$NewBot.PawnClass);

  if ( NewBot.Pawn == None ) {
    log("Couldn't spawn player of type "$NewBot.PawnClass$" at "$StartSpot);
    return;
  } else {
    //   log("Spawned player of type "$NewBot.PawnClass$" at "$StartSpot);
  }

  NewBot.Pawn.Anchor = startSpot;
  NewBot.Pawn.LastStartSpot = PlayerStart(startSpot);
  NewBot.Pawn.LastStartTime = Level.TimeSeconds;
  NewBot.PreviousPawnClass = NewBot.Pawn.Class;

  NewBot.Possess(NewBot.Pawn);
  NewBot.PawnClass = NewBot.Pawn.Class;

  NewBot.Squad = spawn(DeathMatch(Level.Game).DMSquadClass);
  if(NewBot.Squad == None) log("NewBot.Squad == None");

  NewBot.Pawn.PlayTeleportEffect(true, true);
  //	NewBot.ClientSetRotation(NewBot.Pawn.Rotation);
  AddDefaultInventory(NewBot.Pawn);
  TriggerEvent( StartSpot.Event, StartSpot, NewBot.Pawn );

   //makes the bots awesommmmee and hopefully helps them send the PLR message in RemoteBot.CheckVision 1-20-06
 //peripheral vision value is from the specs in Bot.uc - Tim Garwood
  //note: you can change any of the following parameters depending on how tough you want the bots to be
  //and how interesting you want the matches to be.
  newbot.skill = 100;

  //bots field of vision
  newbot.pawn.PeripheralVision = -0.2;
  //bots are crazy accurate
  newbot.accuracy = 1.0;
  //bots can strafe
  newbot.strafingability = 1.0;
  //bots shoot ahead of their targets
  newBot.bLeadTarget = true;

 /*
 Revision 1-9-06  Tim Garwood
 Changed the call from BroadcastLocalizedMessage to Level.Game.Broadcast so that the
 right message shows up when someone enters the game.
 */
  Level.Game.Broadcast( newBot, NewBot.PlayerReplicationInfo.PlayerName $" has entered the game.");
  NewBot.GotoState('StartUp', 'Begin');
}



/* Revision 1-16-06  Tim Garwood
Try using super class killed function.
There really shouldn't be anything special about how a RemoteBot dies. just need to send a message.
*/

function Killed( Controller Killer, Controller Killed, Pawn KilledPawn, class<DamageType> damageType )
{


  local string ib, as, ae;
  local Controller C;

  ib = theBotServer.ib;
  as = theBotServer.as;
  ae = theBotServer.ae;

    if( Killed.isA('RemoteBot')) {
        RemoteBot(Killed).myConnection.SendLine("DIE" $ib$as$ "Killer" $ib$ Killer $ Killer.PlayerReplicationInfo.PlayerID 
                                                      $ ae$ib$as$ "DamageType" $ib$ damageType $ae);
    }
    
    //REVISION 2-20-06  Tim Garwood
    //this is kind of backwards, but this is where we will send the "KIL" message from since
    //the engine no longer calls the Killed/Died function in RemoteBot.

    for( C = Level.ControllerList; C != none; C = C.NextController) {

      if( C.isA('RemoteBot') && C != Killed) {

        RemoteBot(C).myConnection.sendLine("KIL" $ib$as$ "Id" $ib$ Killed $ Killed.PlayerReplicationInfo.PlayerID $ae$ib$as$
        "Killer" $ib$ Killer $ Killer.PlayerReplicationInfo.PlayerID $ae$ib$as$
        "DamageType" $ib$ damageType $ae);

      }

    }

  Super.Killed(Killer, Killed, KilledPawn, damageType);

}

//Called when a controller for this game exits
// prevents a comptuer bot from replacing the remote bot
function Logout(controller Exiting) {
  if(!exiting.IsA('RemoteBot')) {
    Super.Logout(Exiting);
  } else {
    Super(GameInfo).Logout(Exiting);
    NumRemoteBots--;
    if(NeedPlayers() && !AddBot()) {
      RemainingBots++;
    }
  }
}

////////////////////
//
// Default properties
//
////////////////////

// [work] maybe commented out stuff has to exist for menu stuff???
//BotMenuType="BotAPI.RemoteUTBotConfigSClient"
//GameName="Remote Bot DeathMatch"

defaultproperties
{

   ScoreBoardType="XInterface.ScoreBoardDeathMatch"
   HUDType="XInterface.HudBDeathMatch"
   MapListType="XInterface.MapListDeathMatch"
   MapPrefix="DM"
   DefaultEnemyRosterClass="XGame.xDMRoster"
   bPauseable=false
   GameName="Remote Bot DeathMatch"
   NetWait=2
   CountDown=0
}
