/*
The Unreal Engine is protected under the GPL.
For more information, please see http://www.epicgames.com/ut2k4_eula.html

Use and distribution of Gamebots is dictated by the terms in the GNU GPL.
http://www.gnu.org/licenses/gpl.html

Copyright 2006,  Joe Manolovich, Jessica Bayliss, Tim Garwood
*/

/*
 BotDoubleDomGame.uc
 Double Domination game...with bots!
*/

class BotDoubleDomGame extends xDoubleDom
      config(BotAPI);

////////////////////
//
// variables
//
////////////////////

var VizServer 		    theVizServer;
var BotServer 		    theBotServer;
var bool 		    bServerLoaded;
var RemoteBotInfo           RemoteBotConfig;
var int 		    NumRemoteBots;
var GameTypeHelper          game;
// it appears that 2 is the max teams now
var int maxTeams;

// mc - localized PlayInfo descriptions & extra info
var private localized string DDomPropsDisplayText[2];

////////////////////
//
// Functions
//
////////////////////

// done
// automagically called
function PostBeginPlay()
{
  Super.PostBeginPlay();
  // set up the two servers
  if(!bServerLoaded) {
      game = spawn(class'GameTypeHelper',self);
      game.setupGameTypeHelper("BotDoubleDomGame");
      theVizServer = spawn(class'VizServer',self);
      theVizServer.gameClass = "BotDoubleDomGame";
      theBotServer = spawn(class'BotServer',self);
      theBotServer.gameClass ="BotDoubleDomGame" ;

      bServerLoaded = true;
    }

  // set up the config for remote bots
  RemoteBotConfig = spawn(class'RemoteBotInfo');
  // max # of teams is now 2
  maxTeams = 2;
  // [work]
  //RemoteBotConfig.Difficulty = BotConfig.Difficulty;
}


// [work?]
// returns a string of game information
function string GetGameInfo()
{
  return game.GetGameInfo( GoalScore, RemainingTime, maxTeams,
			   MaxTeamSize, theBotServer);
}

// done
// return a formatted string of the game status
function string GetGameStatus() {
  local string ib, as, ae;
   local string outStr;
   local int i, j, teamNum;

   ib = theBotServer.ib;
   as = theBotServer.as;
   ae = theBotServer.ae;

   // the status common to all game types
   outStr = "GAM";
   outStr = (outstr $ib$ game.GetPlayerScores( Level.ControllerList, theBotServer  ) );

   // add team info to the message
   outStr = ( outstr $ ib $ as $ "TeamScores" );
   for (i=0;i<MaxTeams;i++)
   {
     outStr = ( outStr $ib$as$ Teams[i].TeamIndex $ib$ Teams[i].Score $ae );
   }
   outStr = ( outStr $ae );

   // Domination specific information
   outStr = ( outStr $ ib $ as $ "DomPoints" );
   //ut2003 change - only 2 dom points this time, not 16 - Tim Garwood 2-6-06
   for ( j=0; j<2; j++ )
     if ( xDomPoints[j] != None )
     {
       if( xDomPoints[j].ControllingTeam == none )
 	teamNum = 255;
       else
 	teamNum = xDomPoints[j].ControllingTeam.TeamIndex;

       outStr = ( outStr $ib$as$ xDomPoints[j] $ib$ teamNum $ae );
     }
   outStr = ( outStr $ae );	 

   return outStr;
}

//Revision 1-17-06  Tim Garwood
//the other RestartPlayer function only spawned the bot if it was in the dying state, but we call
//this function from the dead state. restart the bot regardless of what state it is in.

function RestartPlayer( Controller aPlayer ) {


 if( aPlayer.isA('RemoteBot'))
	SpawnPlayer( RemoteBot(aPlayer));
 else
    Super.RestartPlayer( aPlayer );
}
/*
* Revision 12-8-05  Tim Garwood
Overriding the getnumplayers() method from UnrealMPGameInfo, maybe this
will accurately reflect the number of players in the game

also, added a bunch of accessors/mutators as seen below, i hope they
are self explanatory => they are used in BotConnection.uc when the connection
is closed.
*/

function int getnumplayers() {

	return numplayers;

}

function setnumplayers( int newplayers ) {

	numplayers = newplayers;
	minplayers = numplayers;

}

function int getnumremotebots() {

	return numremotebots;

}

function setnumremotebots( int newbots ) {

	numremotebots = newbots;
	
}

// [work?] changeteam - does it get unbalanced by 1 because
// of the bots like in UT?

// [work?] reduceDamage - disappeared in deathmatch ut2003
// do things work now?

// Add a client bot to the game if possible else return None
function RemoteBot AddRemoteBot(BotConnection theConnection, string clientName, int teamNum, vector startLocation, rotator startRotation, string className) {
  local RemoteBot newBot;
  local NavigationPoint startSpot;
  local int DesiredTeam, i, MinSize;
  local int nextBotTeam;

  //2-7-06  Tim Garwood - used to get the actual bot team # so that bots are actually on the right team! >:0
  local int actualBotTeam;

  startSpot = FindPlayerStart(NewBot);

  // try to spawn a client bot and return immediately if fail
  newBot = SpawnRemoteBot(startSpot, theConnection, startLocation, startRotation);

  if ( newBot == None ) {
    log("Failed to spawn remote bot.");
    return None;
  }

  // increment appropriate numbers since the bot exists
  NumRemoteBots++;
  NumPlayers++;
  //Revision 1-17-06  Tim Garwood
  //added this here so that no server bots enter the game AND so that
  //when a bot dies he doesn't "re-enter" the game, he just gets a new start spot.
  //this is because the engine checks to see if the player who died is a bot and if there
  //are too many players in the game, then it will destroy the bot. we don't want that.
  minplayers = numplayers;

  // set up names and teams
  if(clientName == "") {
    newBot.PlayerReplicationInfo.PlayerName = "" $ newBot;
  } else {
    newBot.PlayerReplicationInfo.PlayerName = clientName;
  }

  if ( bBalanceTeams ) {
    MinSize = Teams[0].Size;
    DesiredTeam = 0;
    for ( i=1; i<maxTeams; i++ )
      if ( Teams[i].Size < MinSize )
      {
	MinSize = Teams[i].Size;
	DesiredTeam = i;
      }
  }
  else
    DesiredTeam = teamNum;

  nextBotTeam = DesiredTeam + 1;
  if ( nextBotTeam >= maxTeams )
      nextBotTeam = 0;


  newBot.PlayerReplicationInfo.bBot = True;


  actualBotTeam = DesiredTeam;

  // note - this wouldn't work if there were more than
  // two possible teams


  if ( (DesiredTeam == 255) || !ChangeTeam(NewBot, DesiredTeam, true) )
  {
    log("Something bad happened in BotDoubleDomGame");

    ChangeTeam(NewBot, nextBotTeam, true);
    actualBotTeam = nextBotTeam;
   }

  if(className != "") {
    newBot.PawnClass = class<Pawn>(DynamicLoadObject(className,class'Class'));
  } else if(newBot.PawnClass == None) {
    log("newBot.PawnClass is None so its and xPawn");
    newBot.PawnClass = class<Pawn>(DynamicLoadObject("xGame.xPawn",class'Class'));
  }

  SpawnPlayer(newBot, startLocation, startRotation);

  return NewBot;
}

// Used by AddRemoteBot - does the actual bot spawning and if fails
// (maybe from a lack of starting
// positions), then returns None
function RemoteBot SpawnRemoteBot(NavigationPoint StartSpot, BotConnection theConnection, vector startLocation, rotator startRotation)
{
  local actor tempbot;
  local RemoteBot newBot;

  //newBot = Spawn(class'BotAPI.RemoteBot',self,,startLocation,startRotation);

  /****  REVISION:   11-30-05     Tim Garwood  ****/
  /****  The above line of code did not compile, it was generating a type mismatch ***/
  /****  so instead just generate a temp bot actor object and then immediately cast ***/
  /**** it into a RemoteBot and store it in newbot **/

  tempBot = Spawn(class'BotAPI.RemoteBot',self,,startLocation,startRotation);
  newbot = RemoteBot( tempbot );
  newBot.bIsPlayer = true;
  newBot.bHidden = false;

  if ( newBot != None )
  {
    //hook up connection to socket
    NewBot.myConnection = theConnection;
    NewBot.as = theConnection.as;
    NewBot.ae = theConnection.ae;
    NewBot.ib = theConnection.ib;
    
    // Set the player's ID.
    NewBot.PlayerReplicationInfo.PlayerID = CurrentID++;

  }

  return NewBot;
}


// [2003 new]
// needs to be in the game rather than the helper to spawn
// correctly
function SpawnPlayer(RemoteBot newBot, optional vector startLocation, optional rotator startRotation ) {
  local NavigationPoint startSpot;

  if(NewBot == None) {
    log("NewBot was none");
    return;
  }

  if(NewBot.Pawn != None) {
    log("Already had pawn");
    return;
  }

  startSpot = FindPlayerStart(NewBot);

  if(startLocation == vect(0,0,0)) {
    newBot.Pawn = Spawn(GetDefaultPlayerClass(NewBot),,,StartSpot.Location,StartSpot.Rotation);
  } else {
     NewBot.Pawn = Spawn(NewBot.PawnClass,,,startLocation,startRotation);
  }

  //  log("PawnClass: "$NewBot.PawnClass);

  if ( NewBot.Pawn == None ) {
    log("Couldn't spawn player of type "$NewBot.PawnClass$" at "$StartSpot);
    return;
  } else {
    //   log("Spawned player of type "$NewBot.PawnClass$" at "$StartSpot);
  }

  NewBot.Pawn.Anchor = startSpot;
  NewBot.Pawn.LastStartSpot = PlayerStart(startSpot);
  NewBot.Pawn.LastStartTime = Level.TimeSeconds;
  NewBot.PreviousPawnClass = NewBot.Pawn.Class;

  NewBot.Possess(NewBot.Pawn);
  NewBot.PawnClass = NewBot.Pawn.Class;


  //NewBot.Squad = spawn(DeathMatch(Level.Game).DMSquadClass);
  //ut2003 change 1-30-06  Tim Garwood
  NewBot.Squad = spawn(TeamGame(Level.Game).DMSquadClass);

  if(NewBot.Squad == None) log("NewBot.Squad == None");

  NewBot.Pawn.PlayTeleportEffect(true, true);
  //	NewBot.ClientSetRotation(NewBot.Pawn.Rotation);
  AddDefaultInventory(NewBot.Pawn);
  TriggerEvent( StartSpot.Event, StartSpot, NewBot.Pawn );
  
  //makes the bots awesommmmee and hopefully helps them send the PLR message in RemoteBot.CheckVision 1-20-06
 //peripheral vision value is from the specs in Bot.uc - Tim Garwood
  //note: you can change any of the following parameters depending on how tough you want the bots to be
  //and how interesting you want the matches to be.
  newbot.skill = 100;

  //bots field of vision
  newbot.pawn.PeripheralVision = -0.2;
  //bots are crazy accurate
  newbot.accuracy = 1.0;
  //bots can strafe
  newbot.strafingability = 1.0;
  //bots shoot ahead of their targets
  newBot.bLeadTarget = true;


  /*
 Revision 1-9-06  Tim Garwood
 Changed the call from BroadcastLocalizedMessage to Level.Game.Broadcast so that the
 right message shows up when someone enters the game.
 */
  Level.Game.Broadcast( newBot, NewBot.PlayerReplicationInfo.PlayerName $" has entered the game.");
  NewBot.GotoState('StartUp', 'Begin');

}


/* Revision 1-16-06  Tim Garwood
Try using super class killed function.
There really shouldn't be anything special about how a RemoteBot dies. just need to send a message.
*/

function Killed( Controller Killer, Controller Killed, Pawn KilledPawn, class<DamageType> damageType )
{


  local string ib, as, ae;
  local Controller C;

  ib = theBotServer.ib;
  as = theBotServer.as;
  ae = theBotServer.ae;

    if( Killed.isA('RemoteBot')) {
        RemoteBot(Killed).myConnection.SendLine("DIE" $ib$as$ "Killer" $ib$ Killer $ Killer.PlayerReplicationInfo.PlayerID 
                                                      $ ae$ib$as$ "DamageType" $ib$ damageType $ae);
    }
    
    //REVISION 2-20-06  Tim Garwood
    //this is kind of backwards, but this is where we will send the "KIL" message from since
    //the engine no longer calls the Killed/Died function in RemoteBot.

    for( C = Level.ControllerList; C != none; C = C.NextController) {

      if( C.isA('RemoteBot') && C != Killed) {

        RemoteBot(C).myConnection.sendLine("KIL" $ib$as$ "Id" $ib$ Killed $ Killed.PlayerReplicationInfo.PlayerID $ae$ib$as$
        "Killer" $ib$ Killer $ Killer.PlayerReplicationInfo.PlayerID $ae$ib$as$
        "DamageType" $ib$ damageType $ae);

      }

    }

  Super.Killed(Killer, Killed, KilledPawn, damageType);

}

////////////////////
//
// Default properties
//
////////////////////

// [work] maybe commented out stuff has to exist for menu stuff???
//BotMenuType="BotAPI.RemoteUTBotConfigSClient"
//GameName="Remote Bot DeathMatch"


State MatchInProgress
{
	function Timer()
	{
		local Controller TeamMate;
		local int i;

		Super(TeamGame).Timer();

		if (bGameEnded || xDomPoints[0] == None || xDomPoints[1] == None)
			return;

		// are both points currently disabled?
		if (DisabledCountDown > 0)
		{
		DisabledCountDown--;
                if ( DisabledCountDown == 7 )
                {
                 for ( TeamMate = Level.ControllerList; TeamMate != None; TeamMate = TeamMate.NextController )
                    if ( PlayerController(TeamMate) != None )
			PlayerController(TeamMate).PlayAnnouncement(NewRoundSound,1,true);
                }
            else if ( (DisabledCountDown > 0) && (DisabledCountDown < 6) )
                 BroadcastLocalizedMessage(class'TimerMessage', DisabledCountDown);

                 if (DisabledCountDown == 0)
		{
		    xDomPoints[0].ResetPoint(true);
		    xDomPoints[1].ResetPoint(true);
		}

            }

            // nothing more to do unless the same team controls both points
            if ( (xDomPoints[0].ControllingTeam != xDomPoints[1].ControllingTeam) || (xDomPoints[0].ControllingTeam == None) )
               return;

		if (ScoreCountDown == TimeToScore)
		{
		    for (TeamMate = Level.ControllerList; TeamMate != None; TeamMate = TeamMate.NextController)
                    {
			    if (TeamMate.IsA('PlayerController'))
				    PlayerController(TeamMate).PlayAnnouncement(ControlSounds[xDomPoints[0].ControllingTeam.TeamIndex],1,true);
 	             }
		}

		if (ScoreCountDown > 0 && ScoreCountDown < 9)
		{
		    for (TeamMate = Level.ControllerList; TeamMate != None; TeamMate = TeamMate.NextController)
                    {
			    BroadcastLocalizedMessage(class'TimerMessage', ScoreCountDown );
        	     }
		}

		if (ScoreCountDown > 0)
                {
                 // decrement the time remaining until a point is scored
	         ScoreCountDown--;
                 return;
                }

		for (TeamMate = Level.ControllerList; TeamMate != None; TeamMate = TeamMate.NextController)
                {
			if (TeamMate.IsA('PlayerController'))
			{
				PlayerController(TeamMate).PlayAnnouncement(ScoreSounds[xDomPoints[0].ControllingTeam.TeamIndex],1,true);
				PlayerController(TeamMate).ClientPlaySound(class'CTFMessage'.Default.Riffs[Rand(3)]);
			}
                }

		// controlling team has scored!
                xDomPoints[0].ControllingTeam.Score += 1.0;

		if (GameStats!=None)
			GameStats.TeamScoreEvent(xDomPoints[0].ControllingTeam.TeamIndex,1,"dom_teamscore");
                // award scoring players some Adrenaline and increase their score
		if ( (xDomPoints[0].ControllingPawn != None) && (xDomPoints[0].ControllingPawn.Controller != None) )
                {
                    xDomPoints[0].ControllingPawn.Controller.PlayerReplicationInfo.Score += 5.0;
		    xDomPoints[0].ControllingPawn.Controller.AwardAdrenaline(ADR_Goal);

			if (GameStats!=None)
				GameStats.ScoreEvent(xDomPoints[0].ControllingPawn.Controller.PlayerReplicationInfo,5,"dom_score");
                 }

		if ( (xDomPoints[1].ControllingPawn != None) && (xDomPoints[1].ControllingPawn.Controller != None) )
                {
                    xDomPoints[1].ControllingPawn.Controller.PlayerReplicationInfo.Score += 5.0;
		    xDomPoints[1].ControllingPawn.Controller.AwardAdrenaline(ADR_Goal);

			if (GameStats!=None)
				GameStats.ScoreEvent(xDomPoints[1].ControllingPawn.Controller.PlayerReplicationInfo,5,"dom_score");

                 }

		// reset count
        ScoreCountDown = TimeToScore;
        
		// make all control points be untouchable for a short period
		DisabledCountDown = TimeDisabled;

		// reset both domination points and disable them
		xDomPoints[0].ResetPoint(false);
		xDomPoints[1].ResetPoint(false);
        
        // check if the game is over because either team has achieved the goal limit
        if ( GoalScore > 0 )
        {
		    for (i = 0; i < 2; i++ )
            {
			    if ( Teams[i].Score >= GoalScore )
                {
				    EndGame(None,"teamscorelimit");
                }
            }
        }
    }
}

//Called when a controller for this game exits
// prevents a comptuer bot from replacing the remote bot
function Logout(controller Exiting) {
  if(!exiting.IsA('RemoteBot')) {
    Super.Logout(Exiting);
  } else {
    Super(GameInfo).Logout(Exiting);
    NumRemoteBots--;
    if(NeedPlayers() && !AddBot()) {
      RemainingBots++;
    }
  }
}

/*
function ClearControl(Controller Other)
{

    local Controller P;
	local PlayerController Player;
    local Pawn Pick;
    
    log("In clear control!!!!!!!!!!!!!!");

	if ((PlayerController(Other) == None) || (Other.PlayerReplicationInfo.Team.TeamIndex == 255))
		return;

	if ((xDomPoints[0].ControllingPawn != Other) || (xDomPoints[0].ControllingPawn != Other))
        return;
    
    // find a suitable teammate
	for(P = Level.ControllerList; P != None; P = P.nextController)
	{
		Player = PlayerController(P);

		if (Player != None && (Player != Other) && (Player.PlayerReplicationInfo.Team == Other.PlayerReplicationInfo.Team))
		{
			Pick = Player.Pawn;
            break;
		}
	}

    // update both control points with the new controlling pawn, since the other one is leaving (if no other suitable
    // teammate was found, then the point will return to neutral)
    if (xDomPoints[0].ControllingPawn == Other)
	{
		xDomPoints[0].ControllingPawn = Pick;
		xDomPoints[0].UpdateStatus();
	}

	if (xDomPoints[1].ControllingPawn == Other)
	{
		xDomPoints[1].ControllingPawn = Pick;
		xDomPoints[1].UpdateStatus();
	}
}


function Logout( Controller Exiting )
{
    ClearControl(Exiting);
    Super.Logout(Exiting);
}

function ResetCount()
{
    local Sound Aversion;
	local Controller TeamMate;
    
    if ( ScoreCountDown < TimeToScore )
    {
        Aversion = AvertedSounds[1];
        if (TimeToScore - ScoreCountDown > (TimeToScore/2))
        {
            if (TimeToScore - ScoreCountDown >= (TimeToScore-1))
            {
                // within one second of a score
                Aversion = AvertedSounds[3];
            }
            else
            {
                // halfway to a score
                Aversion = AvertedSounds[2];
            }
        }
        else if ( TimeToScore - ScoreCountDown < 2 )
			Aversion = AvertedSounds[0];
        
        for (TeamMate = Level.ControllerList; TeamMate != None; TeamMate = TeamMate.NextController)
        {
			if (TeamMate.IsA('PlayerController'))
                PlayerController(TeamMate).PlayAnnouncement(Aversion,2,true);
        }
    }

    ScoreCountDown = TimeToScore;
}

function bool CriticalPlayer(Controller Other)
{
	// In DOM, critical players are anyone within 1024 units of the dom point
	// who have los to it.
	
	if ( (vsize(Other.Pawn.Location - xDomPoints[0].Location) <=1024) || 
	     (vsize(Other.Pawn.Location - xDomPoints[1].Location) <=1024) )
		return true; 

	return Super.CriticalPlayer(Other);
}

static function FillPlayInfo(PlayInfo PlayInfo)
{
	local int i;

	Super.FillPlayInfo(PlayInfo);  // Always begin with calling parent

	PlayInfo.AddSetting("Rules", "TimeToScore",  default.DDomPropsDisplayText[i++], 1, 60, "Text", "3;0:60");
	PlayInfo.AddSetting("Rules", "TimeDisabled", default.DDomPropsDisplayText[i++], 1, 70, "Text", "3;0:60");
}
*/

defaultproperties
{

  DefaultEnemyRosterClass="XGame.xTeamRoster"
  HUDType="XInterface.HudBDoubleDomination"
  bPauseable=false
  GameName="Remote Bot Double Domination"
  NetWait=2
  CountDown=0
}
