/*
The Unreal Engine is protected under the GPL.
For more information, please see http://www.epicgames.com/ut2k4_eula.html

Use and distribution of Gamebots is dictated by the terms in the GNU GPL.
http://www.gnu.org/licenses/gpl.html

Copyright 2006,  Jessica Bayliss
*/

//////////////////////////////////////////
//
// Class description: This is a superclass
// for the view server as well as the bot
// server. This class should never be instantiated
// by itself and should only contain functions
// and variables used by all of its children.
//
// Version: $Id$
//
// Revision: $Log$
//
/////////////////////////////////////////

class BotServerConnection extends TcpLink
                config(BotAPI);
////////////////////
//
// variables
//
////////////////////

var config int ListenPort;
var config int MaxConnections;

var bool bBound;
var config bool bDebug;

var int ConnectionCount;

////////////////////
//
// Events
//
////////////////////

// shouldn't happen - the event should get called in the
// client connection
event ReceivedText( string Text )
{
  if(bDebug)
    log("ReceivedText in server class - "$Text);
}

//should never happen - 
// accepted connections should be forwarded to a botconnection
event Accepted()
{
  if(bDebug)
    log("Accepted connection in Server");
}

// called everytime a new botconnection is spawned
event GainedChild( Actor C )
{
  Super.GainedChild(C);
  ConnectionCount++;
  
  // if too many connections, close down listen.
  if(MaxConnections > 0 && ConnectionCount > MaxConnections && LinkState == STATE_Listening)
    {
      if(bDebug)
	Log("Server: Too many connections - closing down Listen.");
      Close();
    }
}

// called when a connection is shut down
event LostChild( Actor C )
{
  Super.LostChild(C);
  ConnectionCount--;

  // if closed due to too many connections, start listening again.
  if(ConnectionCount <= MaxConnections && LinkState != STATE_Listening)
    {
      if(bDebug)
	Log("Server: Listening again - connections have been closed.");
      Listen();
    }
}

////////////////////
//
// Functions
//
////////////////////

// should called by its child classes BeginPlay() functions
function BeginPlay()
{
  Super.BeginPlay();
}

////////////////////
//
// Default properties
//
////////////////////

defaultproperties
{
     ListenPort=3000
     MaxConnections=16
     //Comment: Tim Garwood 12-6-05 
     //every time a BotServerConnection is made it spawns a BotConnection object which
     //does the dirty work??
     //this code is what forwards all events to the BotConnection
     AcceptClass=Class'BotAPI.BotConnection'
}
