/*
The Unreal Engine is protected under the GPL.
For more information, please see http://www.epicgames.com/ut2k4_eula.html

Use and distribution of Gamebots is dictated by the terms in the GNU GPL.
http://www.gnu.org/licenses/gpl.html

Copyright 2006,  Jessica Bayliss, Tim Garwood
*/

//////////////////////////////////////////
//
// Class Description: Contains overlapping
// functionality for different game types since
// the game types themselves have no common
// superclass in the UT hierarchy. The class extends
// from TeamGamePlus in order to have access to all
// appropriate types for games.
//
// Version: $Id: GameTypeHelper.uc,v 1.1 2003/08/01 17:13:18 jdb Exp jdb $
//
// Revision: $Log: GameTypeHelper.uc,v $
// Revision: Revision 1.1  2003/08/01 17:13:18  jdb
// Revision: Initial revision
// Revision:
//
/////////////////////////////////////////

class GameTypeHelper extends Actor;

////////////////////
//
// variables
//
////////////////////
var string                  gameType;

////////////////////
//
// Functions
//
////////////////////

// setup beginning game variables needed by all
// game types
function setupGameTypeHelper( string gtype )
{
  gameType = gtype;
}

// returns a formatted string of game info
function string GetGameInfo( int score, int tLimit, 
			     int mTeams, int mTeamSize, 
			     BotServer theBotServer)
{
  local string ib, as, ae;
  local string outStr;

  ib = theBotServer.ib;
  as = theBotServer.as;
  ae = theBotServer.ae;
  
  outStr = (ib$as$ "GoalTeamScore" $ib$ score $ae$ib$as$
	    "RemainingTime" $ib$ tLimit $ae$ib$as$
	    "MaxTeams" $ib$ mTeams $ae$ib$as$
	    "MaxTeamSize" $ib$ mTeamSize $ae);
  
  return outStr;
}

// returns a formatted string of player scores
function string GetPlayerScores( Controller p, BotServer theBotServer  )
{

  local string ib, as, ae;
  local string outStr;

  ib = theBotServer.ib;
  as = theBotServer.as;
  ae = theBotServer.ae;


  outStr = (ib $ as $ "PlayerScores" );
  for ( p=p; p!=None; p=p.NextController ) {
       //2-2-06  had to add this check here because apparently UT2003 adds
       //a ScriptedTriggerController to the ControllerList in CTF games.
       //since ScriptedTriggerController's aren't Players, their PRI never gets
       //initialized and this loop throws null pointers in CTF games.   -Tim Garwood
     if( p.PlayerReplicationInfo != none ) {

       if( !p.PlayerReplicationInfo.bIsSpectator )
           //updated 2-3-06 to send the unique playerID as part of the string.
	  outStr = ( outStr $ib$as$ p $ p.PlayerReplicationInfo.PlayerID $ ib$ p.PlayerReplicationInfo.Score $ae );

     }
  }

  outStr = ( outStr $ae );

  return outStr;
}




