/*
The Unreal Engine is protected under the GPL.
For more information, please see http://www.epicgames.com/ut2k4_eula.html

Use and distribution of Gamebots is dictated by the terms in the GNU GPL.
http://www.gnu.org/licenses/gpl.html

Copyright 2006,  Jessica Bayliss, Tim Garwood
*/

//////////////////////////////////////////
//
// VizConnection.uc
//
// Version: $Id$
//
// Revision: $Log$
//
/////////////////////////////////////////

class VizConnection extends BotClientConnection;

////////////////////
//
// variables
//
////////////////////

// the one server
var VizServer Parent;

// set true to allow viz clients to pause
var config bool bAllowPause;

// delay between visionUpdates
var config float updateTime;

// record once we have sent navs - don't need to duplicate
var bool sentNavs;

// keeps track of whether we're paused or not independent of having
// players in the game
var bool areWePaused;

////////////////////
//
// Events
//
////////////////////


//Socket established
event Accepted()
{
  local string gameInfoStr, levelName;
  local int i;
	
  log("Viz Connection established.");
  
  if(bDebug)
    log("Accepted VizConnection");
  
  gameInfoStr = getGameStatus(Parent.gameClass);
  levelName = string(Level);
  i = InStr(Caps(levelName), ".LEVELINFO");
  if(i != -1)
    levelName = Left(levelName, i);

  SendLine("NFO" $ib$as$ "Gametype" $ib$ Parent.gameClass $ae$ib$as$ 
	   "Level" $ib$ levelName $ae$ gameInfoStr);

  gotoState('monitoring','Running');
}

//Closed on other end
event Closed()
{
  Destroy();
}

////////////////////
//
// Functions
//
////////////////////

function PostBeginPlay()
{
  Parent = VizServer(Owner);
  areWePaused = false;
  if(bDebug)
    log("Spawned VizConnection");
}

//Here is where we handle incoming commands
/* Commands expected to look like:
runto {Argument value} {Arg value}...
Currently hard coded to take no more than 9 args
Command type and arguments can be
any length, but first space terminates the name. Values can
have spaces or any other kind of character.
*/
function ReceivedLine(string line)
{    
  local string cmdType, argBody, rem;
  local int endloc, wordsplit, attrNum;
  
  

  if(bDebug)
    log(line);

  wordsplit = InStr(line,ib);
  if( wordsplit == -1)
    wordsplit = Len(line);
  
  cmdType = left(line,wordsplit);	
  rem = mid(line,InStr(line,as));
  attrNum = 0;

  //iterate through attr/val pairs, storring them in the
  //parallel arrays ReceivedArgs and ReceivedVals
  while(attrNum < MAX_ARG_NUM && rem != "")
    {
      endloc = InStr(rem,ae);
      argBody = mid(rem,1,(endloc - 1));
      
      wordsplit = InStr(argBody,ib);
      ReceivedArgs[attrNum] = left(argBody,wordsplit);
      ReceivedVals[attrNum] = mid(argBody,(wordsplit + 1));
      
      rem = mid(rem,1); //advance
      rem = mid(rem,InStr(rem,as));
      attrNum++;
    }
  
  cmdType = Caps(cmdType);
  
  ProcessAction(cmdType);
}


// handles pausing and unpausing the whole game
function ProcessAction(string cmdType)
{
  local Controller playerToPause;
  
  if(bDebug)
    log("comandType:"@cmdType);
	
  switch(cmdType)
    {
    case "PAUSE":
      if (bAllowPause) {
	  // we have to blame the pause on somebody - blame the first player
	  playerToPause = Level.ControllerList;
  	  Level.Pauser= playerToPause.PlayerReplicationInfo;
	  areWePaused = true;
	}
	break;
    case "UNPAUSE":
      if (bAllowPause) {
	  Level.Pauser=None;
	  areWePaused = false;
	}
        break;
    }
}

// iterate through objects and send appropriate messages

function checkGame()
{

	local Inventory I;
	local Mover M;
	local Weapon weapon;
	local Pickup pickup;
	local CTFFlag C;
	local string outstring;
	local int temp;


        //added 2-6-06  Tim Garwood - because Team object is 'none' if this is a DeathMatch
        local string team;

        //ut2003 addition - since we send bombing run info now we need these local vars
        local xBombFlag bombFlag;
        local xBombDelivery deliveryHole;


  // [2003 change] Controller is now the interface for a Pawn
  local Controller P;
  local string plrString;
  local NavigationPoint N;
  // [work] can't comment the following in until there are
  // the appropriate game types defined
  //local CTFFlag C;
  //local bool isVisible, sendUpdate;
  //local int repeat, temp;
  //local string navType, outstring;

  local int count;
  count = 0;

  //log("**********Checking game in VizConnection.uc******************");
  //log("********Sending BEG string*************");
  SendLine("BEG" $ib$as$ "Time" $ib$ Level.TimeSeconds $ae);

  // send the game's info
  //2-2-06 - passing none here will cause a null pointer in sendGameStateFromGameType for CTF games.
  //why are we passing 'none'? - TG
    sendGameStateFromGameType(none, Parent.gameClass);

  //
  for ( P=Level.ControllerList; P!=None; P=P.NextController )
    {
	// For some reason bHidden is sometimes set to true even when we can see
	// real players. So for now we're going to ignore it and look at everything.
      //if( !P.bHidden )
	//  {
	    //added 2-6-06  Tim Garwood
            //have to test for gametype because there is no Team object in a BotDeathMatch.
            if( Level.Game.isA('BotDeathMatch')) {

                team = "Team" $ib$ "255";

            } else {

                team = "Team" $ib$ P.PlayerReplicationInfo.Team.TeamIndex;

            }
		//log("sending PLR string from VizConnection.uc");
	    // [work] if teamID the right number to pull out of the hat???
        //  plrString = "PLR" $ib$as$ "Id" $ib$ P.PlayerReplicationInfo.PlayerName $ae$ib$as$

		//Revision 1-13-06  Tim Garwood  - changed this to send the name and ID instead of just the name
			plrString = "PLR" $ib$as$ "Id" $ib$ P $ P.PlayerReplicationInfo.PlayerID $ae$ib$as$
		      "Rotation" $ib$ P.Pawn.Rotation $ae$ib$as$
		      "Location" $ib$ P.Pawn.Location $ae$ib$as$
		      team $ae$ib$as$
		      "Name" $ib$ P.PlayerReplicationInfo.PlayerName $ae$ib$as$
		      "Weapon" $ib$ P.Pawn.Weapon $ae;

	  if( P.bFire != 0 )
	    plrString = plrString $ib$as$ "Firing" $ib$ "1" $ae;
	  else if( P.bAltFire != 0 )
	    plrString = plrString $ib$as$ "AltFiring" $ib$ "2" $ae;
	  else
	    plrString = plrString $ib$as$ "AltFiring" $ib$ "0" $ae;

	  sendLine(plrString);
    //}
  }

  for ( N=Level.NavigationPointList; N!=None; N=N.NextNavigationPoint )
    {

      // [work] commented out code is domination game specific
      //if(N.IsA('ControlPoint'))
      //{
      //[ut2003 change] - Tim Garwood 2-6-06
      //ControlPoint is now called DominationPoint (or xDomPoint which subclasses DominationPoint)
      if( N.isA('xDomPoint')) {
	  if( xDomPoint(N).ControllingTeam == none )
	    temp = 255;
	  else
	    temp = xDomPoint(N).ControllingTeam.TeamIndex;

	  outstring = "DOM" $ib$as$ "Id" $ib$ N $ae$ib$as$
		   "Location" $ib$ N.Location $ae$ib$as$
		   "Controller" $ib$ temp $ae;

          log( outString );
          SendLine( outstring );
	}

	  if( !sentNavs ){
	  SendLine("NAV" $ib$as$ "Id" $ib$ N $ae$ib$as$
		   "Location" $ib$ N.Location $ae);
	  }
	}

  sentNavs = true;
  count = 0;

  /* Revision 1-13-06  Tim Garwood
  with the new ut2003 map, none of the objects are in the 'pickup' state, so
  try checking class 'Pickup' instead?
  this seems to work..
  */
  //foreach AllActors (class'Inventory', I)
  //  {
  foreach AllActors (class'Pickup', pickup) {
      if( pickup.GetStateName() == 'Pickup' && !pickup.bHidden )
	{
	  SendLine("INV" $ib$as$ "Id" $ib$ pickup $ae$ib$as$
		   "Location" $ib$ pickup.Location $ae);
	}
   }
   
   
   //2-24-06  Tim Garwood
   //sending out locations of Movers here also since apparently its difficult for the bots
   //to see Movers in RemoteBot if the movers aren't already moving.
  foreach AllActors( class'Mover', M) {
          SendLine("MOV" $ib$as$ "Id" $ib$ M $ae$ib$as$
				"Location" $ib$ M.Location $ae$ib$as$
				"DamageTrig" $ib$ M.bDamageTriggered $ae$ib$as$
				"Class" $ib$ M.Class $ae);
  }

  foreach AllActors (class'Weapon', weapon) {
	//if(weapon.GetStateName() == 'Pickup' && !weapon.bHidden) {
		outstring = "INV" $ib$as$ "Id" $ib$ weapon $ae$ib$as$
		   "Location" $ib$ weapon.Location $ae;
                SendLine(outstring);
	//}
  }
  
  // [work] the following code is capture the flag specific
  foreach AllActors (class'CTFFlag', C)
    {

      outstring = "FLG" $ib$as$ "Id" $ib$ C $ae$ib$as$
      "Team" $ib$ C.Team.TeamIndex $ae$ib$as$
      "State" $ib$ C.GetStateName() $ae;

      //when a flag is held its location is not updated by engine =(
      if( C.GetStateName() == 'Held')
        outstring = outstring $ib$as$ "Location" $ib$ C.Holder.Location $ae$ib$as$
        "Holder" $ib$ C.Holder $ae;
      else
        outstring = outstring $ib$as$ "Location" $ib$ C.Location $ae;
      SendLine(outstring);
    }
    
    //ut2003 addition - sending the location of xBombDelivery holes and
    //xBombFlags
    
    foreach AllActors (class'xBombFlag', bombFlag) {

            outstring = "BOM" $ib$as$ "Id" $ib$ bombFlag $ae;
				//when a flag is held its location is not updated by engine =(
				if( bombFlag.GetStateName() == 'Held')
					outstring = outstring $ib$as$ "Location" $ib$ bombFlag.Holder.Location $ae$ib$as$
								"Holder" $ib$ bombFlag.Holder $ae;
				else
					outstring = outstring $ib$as$ "Location" $ib$ bombFlag.Location $ae;

				outstring = outstring $ib$as$ "State" $ib$ bombFlag.GetStateName() $ae;
				log( outstring );
            SendLine(outstring);
    }
    
    //ut2003 addition - send out location of bomb deliver holes too.
    foreach AllActors( class'xBombDelivery', deliveryHole ) {

            outstring = "HOL" $ib$as$ "Id" $ib$ deliveryHole.MyHole $ae$ib$as$
		   "Location" $ib$ deliveryHole.MyHole.Location $ae$ib$as$
                   "Team" $ib$ deliveryHole.Team $ ae;

	   log( outstring );
	   SendLine( outstring );

    }

  SendLine("END" $ib$as$ "Time" $ib$ Level.TimeSeconds $ae);
}

////////////////////
//
// States
//
////////////////////

//fire right up into the loop for sending updates
auto state monitoring
{
Begin:
Running:
  if( !areWePaused )
	checkGame();
  sleep(updateTime);
  goto 'Running';
}

defaultproperties
{
     bAllowPause=True
     updateTime=0.200000
     AS="{"
     ae="}"
     ib=" "
}
